//This file is part of the LTakeshi v0.0 library written in -*- C++ -*-

//LTakeshi v0.0: a multi-platform C++ library for GUI management
//Copyright (C) Luca Saiu, 1999-2000

//This is free software; you can redistribute it and/or modify it
//under the terms of the GNU Lesser General Public License version 2.1
//as published by the Free Software Foundation.

//This software is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this file; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//Luca Saiu
//------------------------------------------
//e-mail:      positrone@freemail.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


#ifndef _LATTRIBUTETABLE_H
#define _LATTRIBUTETABLE_H

#include "../Lhash.h"
#include "../Lstring/Lstring.h"
#include "LTypes.h" //for LCallBack type definition

#define TABLESIZE 8

typedef unsigned char LFlags;      // Internally used bit-field of flags for
                                   // attributes

class LAttributeTable{
protected:
  // Hash tables to store data
  Lhash<int,LName,TABLESIZE>       HashInt;
  Lhash<double,LName,TABLESIZE>    HashDouble;
  Lhash<bool,LName,TABLESIZE>      HashBool;
  Lhash<Lstring,LName,TABLESIZE>   HashString;
  Lhash<void*,LName,TABLESIZE>     HashPointer;
  Lhash<LCallBack,LName,TABLESIZE> HashCallBack;
  Lhash<LFlags,LName,TABLESIZE>    HashFlags;
public:
  // Constructor and destructor
  LAttributeTable();
  ~LAttributeTable();

  // InsertXXXX methods:
  void InsertInt(const LName,const bool readonly=false,const int x=0);
  void InsertDouble(const LName,const bool readonly=false,const double x=0.);
  void InsertBool(const LName,const bool readonly=false,const bool x=false);
  void InsertString(const LName,const bool readonly=false,const Lstring x="");
  void InsertPointer(const LName,const bool readonly=false,void *x=0);
  void InsertCallBack(const LName,const bool readonly=false,const LCallBack x=0);

  // GetXXXX methods:
  int       GetInt(const LName);
  double    GetDouble(const LName);
  bool      GetBool(const LName);
  Lstring   GetString(const LName);
  void*     GetPointer(const LName);
  LCallBack GetCallBack(const LName);

  // SetXXXX methods:
  void       SetInt(const LName,int);
  void       SetDouble(const LName,double);
  void       SetBool(const LName,bool);
  void       SetString(const LName,Lstring);
  void       SetPointer(const LName,void*);
  void       SetCallBack(const LName,LCallBack);

  // Interrogation methods:
  bool       Exists(const LName);

  // Flags-handling methods:
  bool       WasModified(const LName);
  void       Reset(const LName);
  bool       TestAndSet(const LName);// Returns the modify flag and clears it
  void       MakeReadOnly(const LName);

  // Debug and error-related methods:
  void       Dump();
};

#endif //#ifndef _LATTRIBUTETABLE_H









