//This file is part of the LTakeshi v0.0 library written in -*- C++ -*-

//LTakeshi v0.0: a multi-platform C++ library for GUI management
//Copyright (C) Luca Saiu, 1999-2000

//This is free software; you can redistribute it and/or modify it
//under the terms of the GNU Lesser General Public License version 2.1
//as published by the Free Software Foundation.

//This software is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this file; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//Luca Saiu
//------------------------------------------
//e-mail:      positrone@freemail.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


#ifndef LCANVAS_H_
#define LCANVAS_H_

#include "LColor.h"
#include "LDrawable.h"
#include "LPixmap.h"

#include "LWidget.h"
#include "sysdep/LCanvas.h"
#include "LWindow.h"

class LCanvas:public LContainer, public LDrawable{
// If this platform needs some 'friend' declarations, the preprocessor
// will put them here
#ifdef PLATFORM_DEPENDANT_LCANVAS_FIELDS
  PLATFORM_DEPENDANT_LCANVAS_FIELDS
#endif
protected:
  //System-dependant field
  SysDepCanvasData SysDepAsCanvas; // Definition is in ./sysdep/LCanvas.h
  LColor CurrentPixelColor;

  //System-dependant methods
  virtual void Apply();            // Commits changes and resets modification
                                   // flags; the definition is in
                                   // ./sysdep/LCanvas.cc, because
                                   // this method is platform-dependant

  virtual void SystemDependantInit();
                                   // Called in the constructor, defined in
                                   // ./sysdep/LCanvas.cc

  virtual void SystemDependantDone();
                                   // Called in the destructor, defined in
                                   // ./sysdep/LCanvas.cc
public:
  // Constructor and Destructor:
  LCanvas(LContainer&);
  ~LCanvas();

  // Public methods:

  // Depending on the implementation, drawing primitives can be just dummy
  // methods calling the ones of the LDrawable base class:
  void Clear();
  void DrawPixel(int x,int y);
  void DrawPixel(int x,int y,const LColor c);
  void DrawLine(int x1,int y1,int x2,int y2);
  void DrawLine(int x1,int y1,int x2,int y2,const LColor c);
  void DrawFullRectangle(int x1,int y1,int x2,int y2);
  void DrawFullRectangle(int x1,int y1,int x2,int y2,const LColor c);
  void DrawFullTriangle(int x1,int y1,int x2,int y2,int x3,int y3);
  void DrawFullTriangle(int x1,int y1,int x2,int y2,int x3,int y3,const LColor c);
  void FastClear();
  void FastDrawPixel(int x,int y);
  void FastDrawPixel(int x,int y,const LColor c);
  void FastDrawLine(int x1,int y1,int x2,int y2);
  void FastDrawLine(int x1,int y1,int x2,int y2,const LColor c);
  void FastDrawFullRectangle(int x1,int y1,int x2,int y2);
  void FastDrawFullRectangle(int x1,int y1,int x2,int y2,const LColor c);
  void FastDrawFullTriangle(int x1,int y1,int x2,int y2,int x3,int y3);
  void FastDrawFullTriangle(int x1,int y1,int x2,int y2,int x3,int y3,const LColor c);

  void SetColor(const LColor c);
  void FastSetColor(const LColor c);
  const LColor GetColor() const;

  void CopyAreaFrom(int x1dest,int y1dest,LPixmap &source,int x1,int y1,int x2,int y1);
  void CopyAreaFrom(int x1dest,int y1dest,LCanvas &source,int x1,int y1,int x2,int y1);

  void FastCopyAreaFrom(int x1dest,int y1dest,LPixmap &source,int x1,int y1,int x2,int y1);
  void FastCopyAreaFrom(int x1dest,int y1dest,LCanvas &source,int x1,int y1,int x2,int y1);
};

#endif//#ifndef LCANVAS_H_

