#include "LColor.h"

extern "C"{
#include <ctype.h> // for isdigit()
}

// Declared as extern in LColor.h:
const int LColor::MaxComponent=65535; // X uses 16-bit integers as RGB
                                      // components.

// Deprecated:
const int MaxLColorComponent=LColor::MaxComponent;

LColor::LColor(){
  // Do nothing
}

LColor::LColor(int r_,int g_,int b_){
  r=r_;
  g=g_;
  b=b_;
}

LColor::LColor(const Lstring s_){
  // Look for separators and divide s into a part for each component:
  int red,green,blue;            // Components
  int from=0,to;                 // loop counters
  Lstring s=s_.NoSpacesAround(); // Temporary string variable

  // Look for the first separator:
  for(to=from;(to<s.getdim())&&isdigit(s[to]);to++);
  // Now i points to the first separator.
  red=s.FromTo(from,to-1).ToLong();
  // Look for the first digit:
  for(from=to;(from<s.getdim())&&!isdigit(s[from]);from++);

  // Look for the first separator:
  for(to=from;(to<s.getdim())&&isdigit(s[to]);to++);
  // Now i points to the first separator.
  green=s.FromTo(from,to-1).ToLong();
  // Look for the first digit:
  for(from=to;(from<s.getdim())&&!isdigit(s[from]);from++);

  // Look for the first separator:
  for(to=from;(to<s.getdim())&&isdigit(s[to]);to++);
  // Now i points to the first separator.
  blue=s.FromTo(from,to-1).ToLong();

  // Assign values to LColor fields:
  r=red;
  g=green;
  b=blue;
}

LColor::LColor(const LColor &x){
  r=x.r;
  g=x.g;
  b=x.b;
}

const LColor LColor::Darken(double ratio) const{
  // Modify colors:
  int red=int(r*(1.-ratio));
  int green=int(g*(1.-ratio));
  int blue=int(b*(1.-ratio));

  // In this case undeflow is impossible. Good.

  // Return the a value:
  return LColor(red,green,blue);
}

const LColor LColor::Lighten(double ratio) const{
  // Modify colors:
  int red=int(r*(1.+ratio));
  int green=int(g*(1.+ratio));
  int blue=int(b*(1.+ratio));

  // Avoid 'overflow':
  if(red>MaxLColorComponent)
    red=MaxLColorComponent;
  if(green>MaxLColorComponent)
    green=MaxLColorComponent;
  if(blue>MaxLColorComponent)
    blue=MaxLColorComponent;

  // Return the a value:
  return LColor(red,green,blue);
}

LColor& LColor::operator=(const LColor x){
  // Copy r, g and b:
  r=x.r;
  g=x.g;
  b=x.b;

  // Return a reference to yourself:
  return *this;
}

const LColor LColorRed=LColor(LColor::MaxComponent,0,0);
const LColor LColorGreen=LColor(0,LColor::MaxComponent,0);
const LColor LColorBlue=LColor(0,0,LColor::MaxComponent);
const LColor LColorCyan=LColor(0,LColor::MaxComponent,LColor::MaxComponent);
const LColor LColorMagenta=LColor(LColor::MaxComponent,0,LColor::MaxComponent);
const LColor LColorYellow=LColor(LColor::MaxComponent,LColor::MaxComponent,0);
const LColor LColorWhite=LColor(LColor::MaxComponent,LColor::MaxComponent,LColor::MaxComponent);
const LColor LColorBlack=LColor(0,0,0);
const LColor LColorGray=LColor(LColor::MaxComponent/2,LColor::MaxComponent/2,LColor::MaxComponent/2);

