//This file is part of the LTakeshi v0.0 library written in -*- C++ -*-

//LTakeshi v0.0: a multi-platform C++ library for GUI management
//Copyright (C) Luca Saiu, 1999-2000

//This is free software; you can redistribute it and/or modify it
//under the terms of the GNU Lesser General Public License version 2.1
//as published by the Free Software Foundation.

//This software is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this file; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//Luca Saiu
//------------------------------------------
//e-mail:      positrone@freemail.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


#include "LCommonDialogs.h"
#include "../Lstring/Lstring.h"
#include "LWidget.h"
#include "LWindow.h"
#include "LTextLabel.h"
#include "LInputBox.h"
#include "LButton.h"
#include "LAttributes.h"

// Define some simple macros
#define DIALOGWIDTH 480
#define BUTTONWIDTH 80

// Auxiliary structure used to keep together the temporary data referred to
// a dialog window.
struct LCommonDialog{
  // Fields
  LButton*   buttons[5];
  LCallBack  callbacks[5];
  LWindow    window;
  LTextLabel textlabel;

  // Constructor and destructor
  LCommonDialog();
  ~LCommonDialog();
};

LCommonDialog::LCommonDialog():textlabel(window){
  int i; // Loop counter

  // 'Void' buttons should point to nothing: in this way it's easy to free
  // only the really used storage.
  for(i=0;i<5;i++)
    buttons[i]=0;
}

LCommonDialog::~LCommonDialog(){
  int i; // Loop counter

  // Delete used buttons
  for(i=0;i<5;i++)
    if(buttons[i]!=0)
      delete buttons[i];
  // textlabel and window are implicitally destroyed.
}

void ShowMessage(Lstring Text){
  // It's trivial: simply call ChoiceDialog()
  ChoiceDialog(Text,"Ok");
}

// The effect of the 'Ok' button or of the choice-buttons
void MessageEffect(LWidget &me){
  // Find the LCommonDialog structure
  LCommonDialog* cd=(LCommonDialog*)(me.GetPointer(LAUserDefinedData));

  // Dirty trick: find the right callback from the number of the button
  // stored in the LName attribute
  LCallBack cb=cd->callbacks[me.GetString(LAName).ToLong()];

  // Call the cb callback, if it's not null
  if(cb!=0)
    cb(me,LVoidEvent);

  // Delete the LCommonDialog structure, calling its destructor and freeing
  // the storage.
  delete cd;
}

// Using default parameters is in general a better solution than a variadic
// function; moreover, in this case it would be really crazy to use more than
// five buttons.
void ChoiceDialog(Lstring text,Lstring button0,LCallBack c0,
                               Lstring button1,LCallBack c1,
                               Lstring button2,LCallBack c2,
                               Lstring button3,LCallBack c3,
                               Lstring button4,LCallBack c4){
  // Using many strings is more natural than using a string vector when
  // calling a function, but it's better to use an array internally:
  // nothing to do, C++ will never be Lisp 8-).

  LArray< Lstring* > buttons;    // Internally-used names array
  int i; // Loop counter

  // Initialize buttons
  buttons.SetDim(5);
  buttons[0]=&button0;
  buttons[1]=&button1;
  buttons[2]=&button2;
  buttons[3]=&button3;
  buttons[4]=&button4;

  // Create a new istance of dialog
  LCommonDialog *ThisDialog=new LCommonDialog;

  // Initialize callbacks
  ThisDialog->callbacks[0]=c0;
  ThisDialog->callbacks[1]=c1;
  ThisDialog->callbacks[2]=c2;
  ThisDialog->callbacks[3]=c3;
  ThisDialog->callbacks[4]=c4;

  // Set window attributes
  ThisDialog->window.Set(LATitle,	"Dialog",
        	 	 LAModal,	true,
       			 LAW,		DIALOGWIDTH,
         		 LAH,		100,
        		 LAEnd);

  // Set text label attributes
  ThisDialog->textlabel.Set(LAX,	10,
         		    LAY,	10,
		            LAText,	text.ToCharStar(),
		            LAEnd);

  // Define data for buttons
  // Count buttons: if the (n+1)-th does not exists but the n-th does,
  // then assume all buttons in [0,n] exist. Assume then at least button0
  // exists.
  int HowManyButtons=1;

  for(i=4;i>0;i--)
    if((*(buttons[i]))!=""){
      HowManyButtons=i+1;
      break; // We have already found the number of button: exit from loop
    }

  // Create buttons, from #0 to #(HowManyButtons-1)
  for(i=0;i<HowManyButtons;i++){
    // Create a new LButton object
    ThisDialog->buttons[i]=new LButton(ThisDialog->window);

    // Set its attributs
    ThisDialog->buttons[i]->Set(LAText,		  (*(buttons[i])).ToCharStar(),
                                // x coordinates for buttons can be found
                                // with this formula: it's simple math.
  			        LAX,		  (DIALOGWIDTH-BUTTONWIDTH*HowManyButtons)/(HowManyButtons+1)*(i+1)+i*BUTTONWIDTH,
			        LAY,		  70,
			        LAW,		  BUTTONWIDTH,
			        LAEffect,	  MessageEffect,
			        LAUserDefinedData,ThisDialog,
                                // Very dirty trick: store the number of the
                                // button in the LAName attribute. However
                                // such tricks will never be necessary when
                                // using static windows. Other than in
                                // LCommonDialogs you should never need such
                                // hacks.
                                LAName,		  Lstring(i).ToCharStar(),
                                LAEnd);
  }

  // Show the window, then return
  ThisDialog->window.Show();
  return;
}









