//This file is part of the LTakeshi v0.0 library written in -*- C++ -*-

//LTakeshi v0.0: a multi-platform C++ library for GUI management
//Copyright (C) Luca Saiu, 1999-2000

//This is free software; you can redistribute it and/or modify it
//under the terms of the GNU Lesser General Public License version 2.1
//as published by the Free Software Foundation.

//This software is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this file; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//Luca Saiu
//------------------------------------------
//e-mail:      positrone@freemail.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


#include "LMenu.h"
#include "LAttributes.h"

extern "C"{
#include <string.h>
}

LMenu::LMenu(LWindow &w,Lstring caption){
  // Set the parent
  parent=&w;

  // There are no modifications to apply at creation time
  modifications=last=0;

  // Create attributes
  InsertString(LAText,false);
  SetString(LAText,caption);

  // Do the system-dependant part of initialization
  SystemDependantInit();       // this also calls Apply()
}

LMenu::~LMenu(){
  // Do the system-dependant part of destruction
  SystemDependantDone();
}

void LMenu::Insert(char *Item,...){
  va_list l;             // used for variadic arguments
  LCallBack callback;    // The callback function bound to Item

  va_start(l,Item); // Item is unuseful here, I wrote it just to
                    // follow the ANSI C++ Standard.

  while(LName(Item)!=LAEnd){ // if this was the last argument then exit now
    // If Item is not a separator then get the variadic callback argument
    if(strcmp(Item,"-"))
      callback=va_arg(l,LCallBack);

    // Insert <Item,callback> into modifications: if Item is "-" then
    // the callback is not meaningful, but this doesn't matter
    if(modifications==0)               // if the list is empty...
                                       // ...create the first element
      modifications=last=(LMenuList*)xmalloc(sizeof(LMenuList));
    else                               // but if it's not empty...
                                       // ... just append
      last=last->Next=(LMenuList*)xmalloc(sizeof(LMenuList));

    // Now last points to the newly-allocated element; store data into it
    last->ItemName=Item;
    last->ItemCallBack=callback;
    last->Next=0;

    // Update Item for the next cycle of the 'while' loop
    Item=va_arg(l,char*);
  } // while

  // Every argument was processed: exit
  va_end(l);

  // Add menu entries read from the list: this is system dependant, so it's
  // done in LMenu::Apply()
  Apply();
  return;
}








