//This file is part of the LTakeshi v0.0 library written in -*- C++ -*-

//LTakeshi v0.0: a multi-platform C++ library for GUI management
//Copyright (C) Luca Saiu, 1999-2000

//This is free software; you can redistribute it and/or modify it
//under the terms of the GNU Lesser General Public License version 2.1
//as published by the Free Software Foundation.

//This software is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this file; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//Luca Saiu
//------------------------------------------
//e-mail:      positrone@freemail.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


#ifndef LMENU_H_
#define LMENU_H_

#include "sysdep/LMenu.h"

#include "LWidget.h"
#include "LWindow.h"

#include "../Lstring/Lstring.h"

// Internally-used linked list structure
struct LMenuList{
  char      *ItemName;
  LCallBack ItemCallBack;
  LMenuList  *Next;
};

class LMenu:public LWidget{
protected:
// If this platform needs some 'friend' declarations, the preprocessor
// will put them here
#ifdef PLATFORM_DEPENDANT_LMENU_FRIENDS
  PLATFORM_DEPENDANT_LMENU_FRIENDS
#endif

  // This linked list keeps track of the menu-entries to be inserted
  LMenuList *modifications;
  LMenuList *last;

  // System-dependant fields: this also includes LTakeshi callbacks bindings
  // to menu entries
  SysDepMenuData SysDep;

  // System-dependant methods
  virtual void Apply();            // Commits changes and resets modification
                                   // flags; the definition is in
                                   // ./sysdep/Menu.cc, because
                                   // this method is platform-dependant

  virtual void SystemDependantInit();
                                   // Called in the constructor, defined in
                                   // ./sysdep/LMenu.cc

  virtual void SystemDependantDone();
                                   // Called in the destructor, defined in
                                   // ./sysdep/LMenu.cc

  LWindow *parent;                 // An LMenu can only be child of a LWindow,
                                   // any container wouldn't work
public:
  // Constructor and destructor
  LMenu(LWindow &w,Lstring caption="Menu");
  ~LMenu();

  // Methods: parameters are char* instead of Lstring's, because in every
  // case I can imagine only constant character strings can be used as menu
  // names.
  void Insert(char *FirstItem,...);
  void Enable(char*);
  void Disable(char*);
};

#endif //#ifndef LMENU_H_






