//This file is part of the LTakeshi v0.0 library written in -*- C++ -*-

//LTakeshi v0.0: a multi-platform C++ library for GUI management
//Copyright (C) Luca Saiu, 1999-2000

//This is free software; you can redistribute it and/or modify it
//under the terms of the GNU Lesser General Public License version 2.1
//as published by the Free Software Foundation.

//This software is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this file; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//Luca Saiu
//------------------------------------------
//e-mail:      positrone@freemail.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


#ifndef _LOBJECT_H
#define _LOBJECT_H

#include "../Lstring/Lstring.h"
#include "LAttributeTable.h"
#include "sysdep/LSysDep.h"        // Declaration of SystemDependantData

// I need to declare the LContainer class, bedacuse there's a LContainer*
// member in LWidget:
class LContainer;

class LWidget:protected LAttributeTable{
protected:
  LContainer *parent;              // Pointer to the container LTakeshi
                                   // widget
 
  // System-dependant field
                                   // At this point derived classes should
                                   // declare their system-dependant data
                                   // field: the definition should be in
                                   // ./sysdep/WIDGETNAME.h; the suggested
                                   // name is SysDepWIDGETNAMEData SysDep

  // System-dependant methods
  virtual void Apply()=0;          // Commits changes and resets modification
                                   // flags; the definition is in
                                   // ./sysdep/WIDGETNAME.cc, because
                                   // this method is platform-dependant

  virtual void SystemDependantInit()=0;
                                   // Called in the constructor of the
                                   // derived class, it is defined in
                                   // ./sysdep/WIDGETNAME.cc

  virtual void SystemDependantDone()=0;
                                   // Called in the destructor of the
                                   // derived class, it is defined in
                                   // ./sysdep/WIDGETNAME.cc

public:
  // Constructor and destructor
  LWidget();
  ~LWidget();

  // Get() and Set()
  void Get(LName,...);  // Queries the attributes
  void Set(LName,...);  // Changes the values of the attributes

  // GetXXXX methods: their semantic is identical to the one of
  // LAttributeTable::GetXXXX methods.
  int       GetInt(const LName);
  double    GetDouble(const LName);
  bool      GetBool(const LName);
  Lstring   GetString(const LName);
  void*     GetPointer(const LName);
  LCallBack GetCallBack(const LName);

  // Other common methods
  LContainer &GetParent(); // Returns a reference to the parent widget
};

#endif //#ifndef _LOBJECT_H



