#include "LSysDep.h"

#include "LDrawable.h"
#include "../../LDrawable.h"

#include "LCanvas.h"
#include "../../LCanvas.h"

#include "LPixmap.h"
#include "../../LPixmap.h"

extern "C"{
#include <X11/Xlib.h>
}

LPixmap::LPixmap(){
  // Create the pixmap:
  drawable=
    XCreatePixmap(GlobalSysDep.display,              // display
                  DefaultRootWindow(GlobalSysDep.display), // drawable used to
                                                     // determine the screen
                  w=1,h=1,                           // width / height
                  DefaultDepth(GlobalSysDep.display,
                               GlobalSysDep.screen)  // depth
                 );
}

LPixmap::LPixmap(const LPixmap &x){
  // Copy the value of parameters:
  w=x.w;
  h=x.h;

  // Create the pixmap:
  drawable=
    XCreatePixmap(GlobalSysDep.display,              // display
                  DefaultRootWindow(GlobalSysDep.display), // drawable used to
                                                     // determine the screen
                  w,h,                               // width / height
                  DefaultDepth(GlobalSysDep.display,
                               GlobalSysDep.screen)  // depth
                 );
}

LPixmap::~LPixmap(){
  // Destroy the pixmap:
  XFreePixmap(GlobalSysDep.display,              // display
              drawable                           // the pixmap to free
             );
}

void LPixmap::Clear(){
  // Dummy method just calling LDrawable.
  LDrawable::Clear();
}

void LPixmap::DrawPixel(int x,int y){
  // Dummy method just calling LDrawable.
  LDrawable::DrawPixel(x,y);
}

void LPixmap::DrawPixel(int x,int y,const LColor c){
  // Dummy method just calling LDrawable.
  LDrawable::DrawPixel(x,y,c);
}

void LPixmap::DrawLine(int x1,int y1,int x2,int y2){
  // Dummy method just calling LDrawable.
  LDrawable::DrawLine(x1,y1,x2,y2);
}

void LPixmap::DrawLine(int x1,int y1,int x2,int y2,const LColor c){
  // Dummy method just calling LDrawable.
  LDrawable::DrawLine(x1,y1,x2,y2,c);
}

void LPixmap::DrawFullRectangle(int x1,int y1,int x2,int y2){
  // Dummy method just calling LDrawable.
  LDrawable::DrawFullRectangle(x1,y1,x2,y2);
}

void LPixmap::DrawFullRectangle(int x1,int y1,int x2,int y2,const LColor c){
  // Dummy method just calling LDrawable.
  LDrawable::DrawFullRectangle(x1,y1,x2,y2,c);
}

void LPixmap::DrawFullTriangle(int x1,int y1,int x2,int y2,int x3,int y3){
  // Dummy method just calling LDrawable.
  LDrawable::DrawFullTriangle(x1,y1,x2,y2,x3,y3);
}

void LPixmap::DrawFullTriangle(int x1,int y1,int x2,int y2,int x3,int y3,const LColor c){
  // Dummy method just calling LDrawable.
  LDrawable::DrawFullTriangle(x1,y1,x2,y2,x3,y3,c);
}

void LPixmap::SetColor(const LColor c){
  // Dummy method calling the LDrawable parent class version:
  LDrawable::SetColor(c);
}

const LColor LPixmap::GetColor() const{
  // Dummy method calling the LDrawable parent class version:
  return LDrawable::GetColor();
}


void LPixmap::Resize(int w_,int h_){
  // Update w and h:
  w=w_;
  h=h_;

  // Destroy the current X Pixmap...
  XFreePixmap(GlobalSysDep.display,              // display
              drawable                           // the pixmap to free
             );
  
  //... then create a new one:
  drawable=
    XCreatePixmap(GlobalSysDep.display,              // display
                  DefaultRootWindow(GlobalSysDep.display), // drawable used to
                                                     // determine the screen
                  w,h,                               // width, height
                  DefaultDepth(GlobalSysDep.display,
                               GlobalSysDep.screen)  // depth
                 );
}

int LPixmap::GetWidth() const{
  // Trivial access method:
  return w;
}

int LPixmap::GetHeight() const{
  // Trivial access method:
  return h;
}

void LPixmap::CopyAreaFrom(int x1dest,int y1dest,
                           LPixmap &source,int x1,int y1,int x2,int y2){
  // Dummy method calling the LDrawable parent class version:
  LDrawable::CopyAreaFrom(x1dest,y1dest,source,x1,y1,x2,y2);
}

void LPixmap::CopyAreaFrom(int x1dest,int y1dest,
                           LCanvas &source,int x1,int y1,int x2,int y2){
  // Dummy method calling the LDrawable parent class version:
  if(source.drawable=XtWindow(source.SysDep.area))
    LDrawable::CopyAreaFrom(x1dest,y1dest,source,x1,y1,x2,y2);
}

