//This file is part of the LTakeshi v0.0 library written in -*- C++ -*-

//LTakeshi v0.0: a multi-platform C++ library for GUI management
//Copyright (C) Luca Saiu, 1999-2000

//This is free software; you can redistribute it and/or modify it
//under the terms of the GNU Lesser General Public License version 2.1
//as published by the Free Software Foundation.

//This software is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this file; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//Luca Saiu
//------------------------------------------
//e-mail:      positrone@freemail.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


#include "../../LScrollBar.h"
#include "../../LAttributes.h"
#include "LSysDep.h"

extern "C"{
#include <math.h> /* for abs() */
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Scrollbar.h>
#include <X11/Xaw/Form.h>
}

static void UniversalLScrollBarCallBackChange(Widget w,XtPointer clientData,
  XtPointer position){ // to be cast to int
  // This dummy callback calls the real LCallBack function, which can be
  // found thru callData, which in its turn contains a pointer to the
  // LTakeshi widget:

  // Find the callback:
  LCallBack c=((LScrollBar*)clientData)->GetCallBack(LAOnChange);

  // Move the thumb: the Athena widget requires it's done manually, in a
  // callback like this; the XawScrollbarSetThumb() convenience routine
  // doesn't seem to work properly, or I haven't understood how to use it
  // (please let me know if you can): so I must set the resource manually,
  // and this is painful because the type is float, and I must avoid it to
  // be cast to double: following the directions in the Athena documentaion
  // for X11R6.3 [C. D. Peterson, "Athena Widget Set - C Language Interface",
  // page 45] I came to this solution:
  float fraction=((float)abs((int)position))/
                 ((LScrollBar*)clientData)->SysDep.length;
  Arg arguments[1];
  XtArgVal *pointer=(XtArgVal*)&fraction;
  XtSetArg(arguments[0],XtNtopOfThumb,*pointer);
  XtSetValues(w,arguments,1);

  cerr<<fraction<<endl;
  cerr<<((LScrollBar*)clientData)->SysDep.length<<endl;
  cerr<<((LScrollBar*)clientData)->SysDep.thumbsize<<endl;
  // If the callback is not null then call it:
  if(c!=0)
    c(*(LScrollBar*)clientData,LVoidEvent);
}

static void UniversalLScrollBarCallBackScroll(Widget w,XtPointer clientData,
  XtPointer position){
  // This dummy callback simply calls the real LCallBack function, which
  // can be found thru callData, which in its turn contains a pointer to the
  // LTakeshi widget:
  LCallBack c=((LScrollBar*)clientData)->GetCallBack(LAOnScroll);

/*  float t,h;
  XtVaGetValues(w,
		XtNtopOfThumb, &t,
                XtNshown,      &h,
                0);
  cerr<<"[topOfThumb=="<<t<<", shown=="<<h<<"]\n";
*/

  // If c is not null then call it
  if(c!=0)
    c(*(LScrollBar*)clientData,LVoidEvent);

  cerr<<"["<<((unsigned)position)<<"]";
}

void LScrollBar::Apply(){
  // Apply width/height changes
  if(TestAndSet(LAW))
    XtVaSetValues(SysDep.widget,
                  XtNwidth,          GetInt(LAW),
                  0);
  if(TestAndSet(LAH)){
    XtVaSetValues(SysDep.widget,
                  XtNheight,	     GetInt(LAH),
                  0);

    Dimension t;
    XtVaGetValues(SysDep.widget,
                  XtNminimumThumb,   &t,
                  0);
    SysDep.thumbsize=t;

    SysDep.length=GetInt(LAH);// To do: this will change when buttons are
                              // added and horizontal scrollbar are
                              // supported.
  }

  // Apply x/y position changes
  if(TestAndSet(LAX))
    XtVaSetValues(SysDep.widget,
                  XtNhorizDistance,	GetInt(LAX),
                  0);
  if(TestAndSet(LAY))
    XtVaSetValues(SysDep.widget,
                  XtNvertDistance,	GetInt(LAY),
                  0); // Apply x/y position changes

  // Universal scroll and change Xt callback are called, even if no LTakeshi
  // callback is set: there's anything special to do when LTakeshi callbacks
  // change.
}

void LScrollBar::SystemDependantInit(){
  // Create Athena widgets
  SysDep.widget=XtVaCreateManagedWidget("LScrollBar",
                scrollbarWidgetClass,parent->SysDep.area,
                  XtNbottom,        XtChainTop,
                  XtNtop,           XtChainTop,
                  XtNleft,          XtChainLeft,
                  XtNright,         XtChainLeft,
                  XtNlength,	    1000,
                  XtNhorizDistance, 1,
                  XtNvertDistance,  1,
                  XtNvisual,        GlobalSysDep.visual,
                  XtNcolormap,      GlobalSysDep.colormap,
                  0);

  // Add callbacks. These functions update LAValue and call LTakeshi
  // callbacks when necessary:
  XtAddCallback(SysDep.widget,XtNthumbProc,
                UniversalLScrollBarCallBackScroll,this);
  XtAddCallback(SysDep.widget,XtNscrollProc,
                UniversalLScrollBarCallBackChange,this);

  // Apply changes for the first time:
  Apply();
}

void LScrollBar::SystemDependantDone(){
}




