//This file is part of the LTakeshi v0.0 library written in -*- C++ -*-

//LTakeshi v0.0: a multi-platform C++ library for GUI management
//Copyright (C) Luca Saiu, 1999-2000

//This is free software; you can redistribute it and/or modify it
//under the terms of the GNU Lesser General Public License version 2.1
//as published by the Free Software Foundation.

//This software is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public
//License along with this file; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.


//Luca Saiu
//------------------------------------------
//e-mail:      positrone@freemail.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


#include "LSysDep.h"
#include "../../../Lgen/Lgen.h"
#include <iostream.h>
#include "../../../Lhash.h"

#include "../../LWidget.h"

extern "C"{
#include <math.h>
}

// For a description of this hash table, see the comment in LSysDep.h:

// Hash function definition; it's the identity function (the module is
// computed within the Lhash object, it's not needed here):
INDEX LCanvasHashFunction(Widget w){
  return (INDEX)w;
}

// The definition of WID2WID_DIMENSION and the external declaration of
// this object are in sysdep/LSysDep.h

// Hash table declaration:
Lhash<LWidget*,Widget,WID2WID_DIMENSION>
                                WidgetToLWidgetStar(LCanvasHashFunction);

// SystemDependantGlobalData methods' definitions start here:

SystemDependantGlobalData::SystemDependantGlobalData(){
  int argc=1;                         // To emulate argc and argv, which
  char *argv[1]={"SomeApplication"};  // need to be passed to XtOpenDisplay
                                      // To do: support command-line args

  // I use fallback resources not to be forced to write default files.
  String resourcesList[]={            // Set default resources
    // LWindow attributes:
    "*LWindow.background:           grey",
    "*LWindow.borderWidth:          1",
    "*LWindowSkeleton.background:   grey",
    "*LWindowStatusBar.background:  grey",
    "*LWindowStatusBar.borderWidth: 0",
    "*LWindowMenuBar.background:    grey",
    "*LMenu*.background:            grey",
    "*LMenuButton.background:       grey",

    // Widgets attributes:
    "*LTextLabel.background:        grey",
    "*LInputBox.background:         white",
    "*LButton.background:           light grey",//steel blue",
    "*LScrollBar.background:        light green",
    "*LCanvas.background:           grey",
    "*LCanvas.borderWidth:          0",      // To do: this must be 0
    "*LCanvas.borderColor:          black",  // To do: this is for testing

    // If using Xaw3D, don't show shadow on menu buttons and menu entries:
    "*LMenuButton*borderWidth:      0", // useful when _not_ using Athena3D
    "*LMenuButton*shadowWidth:      0",
    "*SmeBSB*shadowWidth:           0",

    // The SysDepAsWindow.stupid widget must be invisible:
    "*LStupid*background:           grey",

    // This array must be null-terminated:
    0
  };

  cerr<<"Setting-up system-dependant data for the X Window System:\n";
  XtToolkitInitialize();                   // Initialize the X toolkit
  appContext=XtCreateApplicationContext(); // Create application context
  XtAppSetFallbackResources(appContext,resourcesList);
  if((display=XtOpenDisplay(appContext,NULL,NULL,"LTakeshi",0,0,
                        &argc,argv))==0)   // Open the display and make the
    fatal("LTakeshi: cannot open display");// variable available for other
                                           // modules

  // Set the default drawable and screen:
  drawable=RootWindow(display,screen=DefaultScreen(display));
  ScreenStruct=DefaultScreenOfDisplay(display);
/*
  // Create and set the default colormap:
  colormap=XCreateColormap(display,                       // display
                           drawable,                      // drawable
                           DefaultVisual(display,screen), // visual
                           AllocNone                      // alloc
                          );
*/
  // Select optimal Depth and Visual:
  LookForDepthAndVisual();

  cerr<<"done.\n";
}

/*
#define StaticGray		0
#define GrayScale		1
#define StaticColor		2
#define PseudoColor		3
#define TrueColor		4
#define DirectColor		5
*/
const Lstring DisplayClass2String(int c){
  switch(c){
    case StaticGray:
      return "StaticGray";
    case GrayScale:
      return "GrayScale";
    case StaticColor:
      return "StaticColor";
    case PseudoColor:
      return "PseudoColor";
    case TrueColor:
      return "TrueColor";
    case DirectColor:
      return "DirectColor";
    default:
      return "Unknown";
  }
}

void SystemDependantGlobalData::LookForDepthAndVisual(){
  // Look for the 'deepest' Depth:
  cerr<<"  Looking for Depth\'s:\n";
  int i,chosen=0;
  for(i=0;i<ScreenStruct->ndepths;i++){
    cerr<<"("<<ScreenStruct->depths[i].depth<<")...\n";
    if(ScreenStruct->depths[i].depth>ScreenStruct->depths[chosen].depth)
      chosen=i;
  }
  depth=&(ScreenStruct->depths[chosen]);
  cerr<<"\n"<<depth->depth<<"-bit depth.\n";

  cerr<<"  (the default visual class would be "<<
        DisplayClass2String(DefaultVisual(display,screen)->c_class)<<")\n";

  // Look for the best Visual:
  cerr<<"  Looking for Visual\'s: ";
  // Keep a pointer for each visual class, so that we can remember
  // which visual classes are available:
  Visual *StaticGrayVisual=0,*GrayScaleVisual=0,*StaticColorVisual=0,
         *PseudoColorVisual=0,*TrueColorVisual=0,*DirectColorVisual=0;
  int j;
  for(j=-1;j<depth->nvisuals;j++){
    cerr<<".";
    Visual *v;
    
    if(j > -1)
      v=&(depth->visuals[j]);
    else
      v=DefaultVisual(display,screen);

    cerr<<"  Visual #"<<j<<": "<<v->bits_per_rgb<<" bits per RGB,\t"<<DisplayClass2String(v->c_class)<<" class\n";

    // Update the right pointer:
    switch(v->c_class){
      case StaticGray:{   // fixed
        StaticGrayVisual=v;
        break;
      }
      case GrayScale:{    // Changeable
        GrayScaleVisual=v;
        break;
      }
      case StaticColor:{  // fixed
        StaticColorVisual=v;
        break;
      }
      case PseudoColor:{  // Changeable
        PseudoColorVisual=v;
        break;
      }
      case TrueColor:{    // fixed
        TrueColorVisual=v;
        break;
      }
      case DirectColor:{  // Changeable
        DirectColorVisual=v;
        break;
      }
    }// switch(...
  }// for(...

  // Choose the visual, depending on its class; I can assume at least _one_
  // visual exists, and it has a class:
  if(TrueColorVisual!=0){
    visual=TrueColorVisual;
    ComputeTrueColorParameters();
  }
/*  else if(Visual!=0)
    visual=Visual;
  else if(Visual!=0)
    visual=Visual;
  else if(Visual!=0)
    visual=Visual;
  else if(Visual!=0)
    visual=Visual;
  else if(Visual!=0)
    visual=Visual;
  else if(Visual!=0)
    visual=Visual;
  else if(Visual!=0)
    visual=Visual;
  else if(Visual!=0)
    visual=Visual;*/
  else
    fatal("SystemDependantGlobalData::LookForDepthAndVisual(): can\'t support available visuals");

  cerr<<" "<<DisplayClass2String(visual->c_class)<<"\n";

  // Create a colormap:
  colormap=XCreateColormap(display,                       // display
                           drawable,                      // drawable
                           visual,                        // visual
                           AllocNone                      // alloc
                          );

  // Install colormap:
  XInstallColormap(display,
                   colormap);
}

void SystemDependantGlobalData::ComputeTrueColorParameters(){
  // Examine red, green and blue masks to compute once for all the number of
  // places to shift color components, first to the right, then to the left,
  // to be OR-ed into pixel values; see LDrawable::SetColor() in LDrawable.cc
  int n,m;
#ifdef DEBUG
  cerr<<" ("<<visual->bits_per_rgb<<"bits per RGB component)";

  cerr<<"\nRed mask:   ";
  for(n=0;n<32;n++)
    cerr<<bool(visual->red_mask&(1<<(31-n)));

  cerr<<"\nGreen mask: ";
  for(n=0;n<32;n++)
    cerr<<bool(visual->green_mask&(1<<(31-n)));

  cerr<<"\nBlue mask:  ";
  for(n=0;n<32;n++)
    cerr<<bool(visual->blue_mask&(1<<(31-n)));
  cerr<<"\n";
#endif //#ifdef DEBUG

  // Look where the red component begins and ends:
  for(n=0;(n<32)&&!(visual->red_mask&(1<<n));n++);
  for(m=n;(m<32)&&(visual->red_mask&(1<<m));m++);
#ifdef DEBUG
  cerr<<"RED:   "<<n<<", "<<m<<"\n";
#endif //#ifdef DEBUG
  PlacesToRShiftRed=16-m+n;
  PlacesToLShiftRed=n;

  // Look where the green component begins and ends:
  for(n=0;(n<32)&&!(visual->green_mask&(1<<n));n++);
  for(m=n;(m<32)&&(visual->green_mask&(1<<m));m++);
#ifdef DEBUG
  cerr<<"GREEN: "<<n<<", "<<m<<"\n";
#endif //#ifdef DEBUG
  PlacesToRShiftGreen=16-m+n;
  PlacesToLShiftGreen=n;

  // Look where the blue component begins and ends:
  for(n=0;(n<32)&&!(visual->blue_mask&(1<<n));n++);
  for(m=n;(m<32)&&(visual->blue_mask&(1<<m));m++);
#ifdef DEBUG
  cerr<<"BLUE:  "<<n<<", "<<m<<"\n";
#endif //#ifdef DEBUG
  PlacesToRShiftBlue=16-m+n;
  PlacesToLShiftBlue=n;
}

// The one and only SystemDependantGlobalData object:
SystemDependantGlobalData GlobalSysDep; // The constructor will initialize
                                        // the X toolkit

// Other functions' definitions start here:

void TakeshiMainLoop(){
  // This is very easy with Xt...
  XtAppMainLoop(GlobalSysDep.appContext);
}

void LBeep(){
  cerr<<"\a";
}

// Locate the LTakeshi LWidget from the Xt Widget w and call a LTakeshi
// CallBack function associated with it (useful in Xt action procedures):
void CallLCallBack(Widget w,LName CallBackName,const LEvent e){
  // Locate the LTakeshi LWidget from the Xt Widget:
  LWidget *LTakeshiWidget=WidgetToLWidgetStar[w];


  // Take a pointer to the CallBack:
  LCallBack c=(LTakeshiWidget->GetCallBack(CallBackName));

  // Call the LCallBack function:
  if(c!=0)
    c(*LTakeshiWidget,e);
}

LKey keycode2LKey(unsigned int keycode){
  // To do
  return keycode;
}

