//This file is part of the Lstring v1.0 library written in -*- C++ -*-

//Lstring v1.0: a simple string class
//Copyright (C) Luca Saiu, 1999-2000

//This library is free software; you can redistribute it and/or modify it
//under the terms of the GNU Library General Public License version 2
//as published by the Free Software Foundation.

//This library is distributed in the hope that it will be useful, but
//WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Library General Public License for more details.

//You should have received a copy of the GNU Library General Public
//License along with this library; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


//Luca Saiu
//------------------------------------------
//e-mail:      luca78@dada.it
//paper mail:  Via Camporcioni, 15
//             51019 Ponte Buggianese - Pistoia
//             Italy
//telephone:   0572-635477 Italy


#ifndef LSTRING_H_
#define LSTRING_H_
#include "../Lgen/Lgen.h"
#include "../LArray.h"

class Lstring{
protected:
 char* c,zero;
 INDEX dim;//number of needed char's
public:
  // Constructor and Destructors
  Lstring();               // Default constructor
  Lstring(const Lstring&); // Copy constructor
  Lstring(const char);     // Conversion constructor
  Lstring(const int);      // Conversion constructor
  Lstring(const long);     // Conversion constructor
  Lstring(const float);    // Conversion constructor
  Lstring(const double);   // Conversion constructor
  Lstring(const bool);     // Conversion constructor
  Lstring(const char*);    // Conversion constructor
  ~Lstring();              // Destructor

  bool Contains(char c) const;
  int FindSubstring(const Lstring x) const;
                                 // Returns the index of the fist character
                                 // of the substring of *this which matches x,
                                 // or -1.
  const Lstring Replace(const Lstring a,const Lstring b);
                                 // Replaces every occurrence of a with b
                                 // in the string. Returns a copy of *this.
  const Lstring ReplaceInPlace(const Lstring a,const Lstring b) const;
  int Lines() const;
  int LongestLineLength() const;
  const Lstring FromTo(int a,int b) const; // a and b are included
  const Lstring FromLineToLine(int a,int b) const; // a and b are included,
                                                   // the first line is #1
  const Lstring Invert() const;
  const Lstring WordWrap(int n) const;
  Lstring& operator=(Lstring);
  Lstring& operator=(char*);
  const char* ToCharStar() const;
  const bool Tobool() const;   //This is more useful than it seems...
  const long ToLong() const;
  const float ToFloat() const;
  const LArray<Lstring> Tokenize(char separator=' ') const;
  bool operator==(const Lstring) const;
  bool operator!=(const Lstring) const;
  bool operator<(const Lstring) const;
  bool operator<=(const Lstring) const;
  bool operator>(const Lstring) const;
  bool operator>=(const Lstring) const;
  char& operator[](const INDEX);
  const char& operator[](const INDEX) const;
  const Lstring ToUpper() const;
  const Lstring ToLower() const;
  const Lstring NoSpacesBefore() const;
  const Lstring NoSpacesAfter() const;
  const Lstring NoSpacesAround() const;
  Lstring operator--();
  Lstring operator--(int);
  Lstring operator+=(Lstring);
  Lstring operator+=(char);
  friend const Lstring operator+(const Lstring, const Lstring);
  friend Lstring operator+(Lstring,char);
  friend ostream& operator<<(ostream&,Lstring);
  friend istream& operator>>(istream&,Lstring&);
  const int getdim() const;// number of real (not including '\0') char's
  void setdim(const int);  // number of real (not including '\0') char's
};

#endif //#ifndef LSTRING_H_

